@@ -1,85 +1,58 @@
 
-cna <- function (x, type,
-    ordering = NULL, strict = FALSE, outcome = TRUE, exclude = character(0), 
-    con = 1, cov = 1, con.msc = con, 
-    notcols = NULL, rm.const.factors = FALSE, rm.dup.factors = FALSE,  
-    maxstep = c(3, 4, 10), inus.only = only.minimal.msc && only.minimal.asf,
-    only.minimal.msc = TRUE, only.minimal.asf = TRUE, maxSol = 1e6, 
-    suff.only = FALSE, what = if (suff.only) "m" else "ac",
-    cutoff = 0.5, border = c("up", "down", "drop"),
-    details = FALSE, acyclic.only = FALSE, cycle.type = c("factor", "value"), 
-    asf.selection = c("cs", "fs", "none"), verbose = FALSE){
-  # call and type
+cna <- function (x, outcome = TRUE, con = 1, cov = 1, maxstep = c(3, 4, 10), 
+    measures = c("standard consistency", "standard coverage"), 
+    ordering = NULL, strict = FALSE, exclude = character(0), notcols = NULL, 
+    what = if (suff.only) "m" else "ac", details = FALSE, 
+    suff.only = FALSE, acyclic.only = FALSE, cycle.type = c("factor", "value"), 
+    verbose = FALSE, control = NULL, ...){
+
+  # update control  
+  control <- resolveCnaControl(control, .con = con, ..., 
+                               control_input_null = is.null(control))
+  checkCnaParams()
+  measures <- resolveMeasures(measures)
   cl <- match.call()
-  if (!is.null(attr(x, "type"))) type <- attr(x, "type")
-  if (missing(type)) type <- "auto"
 
-  # resolve inus.only in case it contains an inus definition
-  if (is.character(inus.only)){
-    inus.def <- match.arg(inus.only, c("implication", "equivalence"))
-    inus.only <- TRUE
-  } else {
-    stopifnot(is.logical(inus.only))
-    inus.def <- "implication"
-  }
-  
-  # Checking/enforcing compatibility of inus.only with only.minimal.msc and only.minimal.asf
-  if (inus.only){
-    if (!only.minimal.msc) 
-      stop("Calling cna() with inus.only=TRUE and only.minimal.msc=FALSE is not meaningful.")
-    if (!only.minimal.asf) 
-      stop("Calling cna() with inus.only=TRUE and only.minimal.asf=FALSE is not meaningful.")
-  }
-  
-  # Select asf.selection (choices: "none", "cs", "fs")
-  asf.selection <- match.arg(asf.selection)
-    
-  # variable names  
-  ct <- configTable(x, type = type, rm.const.factors = rm.const.factors, 
-                    rm.dup.factors = rm.dup.factors)
-  if (any(!grepl("[[:alpha:]]", names(ct)))) 
-    stop("All column names must contain a letter.")
-  if (anyDuplicated(tolower(names(ct))) != 0L)
-    stop("Inadmissable column names: names must be unique when case is ignored!")
-  names(ct) <- toupper(names(ct))
-
-  # more formal requirements  
+  # config table  
+  ct <- configTable(x, type = control$type, 
+                    rm.const.factors = control$rm.const.factors, 
+                    rm.dup.factors = control$rm.dup.factors)
+  # minimal size for cna
   if (nrow(ct) <= 1 || ncol(ct) <= 1)
     stop("Config table must have at least two rows and two columns.")
+  control$type <- attr(ct, "type")
   
   # details
-  details <- clarify_details(details)
-  if (!suff.only) details <- union("inus", details)
-  details1 <- setdiff(details, c("coherence", "redundant", "cyclic"))
-  if (acyclic.only){
-    details <- union(details, c("cyclic"))
+  details <- resolveDetails(details)
+  if (!control$inus.only){
+    details <- addDetailMeasure(details, "inus", "front")
   }
+  # details for asf and msc
+  details_asf <- removeDetailMeasure(details, c("coherence", "redundant", "cyclic"))
+  details_msc <- removeDetailMeasure(details_asf, "inus")
 
-  # Check con and cov values and reduce them slightly to avoid failing 
-  # to find conditions due to rounding issues
-  if (length(con) != 1 || con < 0 || con > 1 || 
-      length(cov) != 1 || cov < 0 || cov > 1){
-    stop("Invalid input for 'con' or 'cov'")
-  }
+  # Slightly reduce con, cov, con.msc to avoid failing to find conditions 
+  # due to rounding issues
   d.eps <- nrow(ct) * .Machine$double.eps
   con <- max(con - d.eps, 0)
   cov <- max(cov - d.eps, 0)
-
+  control$con.msc <- max(control$con.msc - d.eps, 0)
+  
   # maxstep
   stopifnot(length(maxstep) == 3, maxstep > 0)
   maxstep[1:2] <- pmin(maxstep[1:2], maxstep[3])
 
-  # Define config, uniqueValues, resp_nms, factMat, nVal, valueId, scores
-  border <- match.arg(border)
-  cti <- ctInfo(ct, cutoff = cutoff, border = border)
+  # ctInfo: representation of configTable in a convenient structure for implementation
+  cti <- ctInfo(ct, cutoff = control$cutoff, border = control$border)
+  # call below is only made to produce message where all values are on same side of cutoff:
+  if (control$type == "fs"){
+    invisible(fs2cs(ct, cutoff = control$cutoff, border = control$border, 
+                    verbose = TRUE))
+  }
   vnms <- colnames(cti$scores)
   freqs <- cti$freq
   
-  # setup output object
-  sol <- vector("list", length(cti$resp_nms))
-  names(sol) <- cti$resp_nms
-  
-  # ordering and outcome
+  # ordering, outcome, notcols, exclude
   ordering <- check.ordering(ordering, cti)
   if (isTRUE(outcome)){ 
     outcome <- cti$resp_nms
@@ -92,30 +65,27 @@ cna <- function (x, type,
   }
   if (is.list(ordering) && (strict || length(ordering[[1]]) == 1)) 
     outcome <- setdiff(outcome, ordering[[1]])
-  if (type == "mv" && any(outcome %in% names(ct))){
+  if (control$type == "mv" && any(outcome %in% names(ct))){
     mvnms <- intersect(outcome, names(ct))
     outcome <- union(setdiff(outcome, mvnms), 
                      cti$resp_nms[unlist(cti$config[mvnms], use.names = FALSE)])
   }
   outcome <- intersect(cti$resp_nms, outcome) # order!
-
-  # notcols
-  if (!is.null(notcols)){
-    if (type == "mv") stop("\"notcols\" not applicable if type==\"mv\"")
-    if (length(notcols) == 1L && notcols == "all"){
-      notcols <- names(ct) 
-      if ("ALL" %in% names(ct)) 
-        warning("'notcols=\"all\"' is ambiguous if a variable has name \"ALL\". ",
-                "notcols is applied to _all_ variables.", call. = FALSE)
-    }
-    if (!is.character(notcols)) notcols <- names(ct)[notcols]
-    notcols <- toupper(notcols)
-    if (!all(notcols %in% names(ct))) stop("Wrong specification of 'notcols'")
-  }
-
+  notcols <- resolveNotCols(notcols, names(ct), control$type)
   fVals <- factorValues(cti)
   exclude <- resolveExclude(exclude, outcome, notcols, fVals)
 
+  # setup solutions list
+  sol <- list(
+    msc = emptyCondTbl(condClass = "stdBoolean", 
+                       minimal = !control$only.minimal.msc,
+                       details = getDetailCols(details_msc)),
+    asf = emptyCondTbl(condClass = "stdBoolean", 
+                       minimal = !control$only.minimal.asf,
+                       details = getDetailCols(details_asf)))
+  sol <- rep(list(sol), length(outcome))
+  names(sol) <- outcome
+
   # main loop over outcomes  
   for (zname in outcome){
 
@@ -139,21 +109,20 @@ cna <- function (x, type,
     } else {
       excl_z <- integer(0)
     }
-    minSuff <- findMinSuff(cti, y, poteff, freqs, con.msc, 
-                           maxstep, only.minimal.msc, exclValues = excl_z)
-
-    if (all(m_is.null(minSuff))) next
-    
-    msc <- lapply(minSuff, make.msc, outcome = zname, cti = cti, 
-                  details = details1, suff.only = suff.only)
-    msc <- do.call(rbind, msc)
+    minSuff <- findMinSuff(cti, y, poteff, freqs, maxstep, control, 
+                           exclValues = excl_z, conCovDef = measures$def)
+    n_msc <- sum(vapply(minSuff, NROW, integer(1)))
     if (verbose){
       cat("Outcome ", zname, " (", match(zname, outcome), "/", length(outcome), "):\n  ", 
-          nrow(msc), " msc found\n", sep = "")
+          n_msc, " msc found\n", sep = "")
     }
-    nstars <- gregexpr("*", msc$condition, fixed = TRUE)
-    msc$complexity <- lengths(nstars) + 1L - (vapply(nstars, "[[", integer(1), 1L) == -1L)
-    msc <- msc[order(msc$complexity, -msc$consistency * msc$coverage, msc$condition), , drop = FALSE]
+    if (n_msc == 0) next
+
+    msc <- lapply(minSuff, make.msc, outcome = zname, cti = cti, 
+                  details = details_msc)
+    msc <- do.call(rbind, msc)
+    if (!isTRUE(getOption("dontsort"))) 
+      msc <- msc[order(msc$complexity, -msc$con * msc$cov, msc$condition), , drop = FALSE]
     rownames(msc) <- NULL
     sol[[zname]] <- list(msc = msc)
 
@@ -165,17 +134,17 @@ cna <- function (x, type,
     noMsc <- m_is.null(.conjList)
     .conjList[noMsc] <- lapply(which(noMsc), function(i) matrix(integer(0), 0, i))
 
-    .sol <- findAsf(cti, y, freqs, con, cov, .conjList, maxSol, maxstep, only.minimal.asf, 
+    .sol <- findAsf(cti, y, freqs, con, cov, .conjList, control, 
+                    maxstep = maxstep, conCovDef = measures$def, 
                     verbose = verbose)
-    stopifnot(length(.sol) == 0 || any(vapply(.sol, is.intList, logical(1))))
-      
-    if (length(.sol)){
-      asf <- make.asf(cti, zname, .sol, inus.only, details1, 
-                      asf.selection = asf.selection, verbose = verbose)
-      sol[[c(zname, "asf")]] <- asf
+    n_asf <- length(.sol)
+    if (n_asf > 0){
+      asf <- make.asf(cti, zname, .sol, control, details = details_asf, 
+                      conCovDef = measures$def, verbose = verbose)
+      n_asf <- nrow(asf) # may be less than before due to inus.only
+      sol[[zname]]$asf <- asf
     }
-    if (verbose)
-      cat("  ", if (length(.sol)) NROW(asf) else 0, " asf found\n\n", sep = "")
+    if (verbose) cat("  ", n_asf, " asf found\n\n", sep = "")
   }
 
   out <- structure(list(), class = "cna")
@@ -187,31 +156,32 @@ cna <- function (x, type,
   out$ordering <- ordering
   out$notcols <- notcols
   out$configTable <- ct
-  if (attr(ct, "type") == "fs") out$fsInfo <- list(cutoff = cutoff, border = border)
+  if (attr(ct, "type") == "fs") out$fsInfo <- list(cutoff = control$cutoff, border = control$border)
   out$solution <- sol
-  out$what <- what
+  out$measures <- measures
+  out$what <- resolveWhat(what)
   out$details <- details
-  out[c("con", "cov", "con.msc", "inus.only", "acyclic.only", "cycle.type")] <- 
-    list(con, cov, con.msc, inus.only, acyclic.only, cycle.type)
-  if (out$inus.only) out$inus.only <- inus.def
-
+  out[c("con", "cov", "acyclic.only", "cycle.type")] <- 
+    list(con, cov, acyclic.only, match.arg(cycle.type))
+  out$control <- control
   return(out)
 }
 
 
-findMinSuff <- function(cti, y, poteff, freqs, con.msc, maxstep, only.minimal.msc, 
-                        exclValues = integer(0)){
+findMinSuff <- function(cti, y, poteff, freqs, maxstep, control,
+                        exclValues = integer(0), conCovDef = 1:2){
+  
   if (cti$type == "mv") poteff <- unique(sub("(.+)=.+", "\\1", poteff))
   nsteps <- min(length(poteff), maxstep[1L])
   minSuff <- vector("list", nsteps)
-  if (!only.minimal.msc) minimList <- vector("list", nsteps)
+  if (!control$only.minimal.msc) minimList <- vector("list", nsteps)
   
   for (.step in seq_along(minSuff)){
     # Select .step-fold conditions occurring in the data
     allkfoldConds <- findAllSubsets(cti$valueId, .step, match(poteff, colnames(cti$valueId)), 
                                     exclValues = exclValues)
     if (NROW(allkfoldConds) == 0) next
-    if (!only.minimal.msc){
+    if (!control$only.minimal.msc){
       minimList[[.step]] <- rep(TRUE, nrow(allkfoldConds))
     }
     # Eliminate combinations that contain a simpler combination that has been selected in an earlier step
@@ -219,7 +189,7 @@ findMinSuff <- function(cti, y, poteff, freqs, con.msc, maxstep, only.minimal.ms
       for (k_ in seq_len(.step-1L))
         if (!is.null(minSuff[[k_]])){
           is.minim <- !hasSubsetInM(allkfoldConds, minSuff[[k_]])
-          if (only.minimal.msc){
+          if (control$only.minimal.msc){
             allkfoldConds <- allkfoldConds[is.minim, , drop = FALSE]
           } else if (!all(is.minim)){
             minimList[[.step]][!is.minim] <- FALSE
@@ -230,17 +200,19 @@ findMinSuff <- function(cti, y, poteff, freqs, con.msc, maxstep, only.minimal.ms
     if (nrow(allkfoldConds) == 0) break
     stopifnot(anyDuplicated(allkfoldConds) == 0L)
     # Select sufficient conditions and add them to output list
-    cons <- conj_conCov(allkfoldConds, cti$scores, y, freqs)
-  if (any(isSuff <- cons[1, ] >= con.msc, na.rm = TRUE)){
+    cons <- conj_conCov(allkfoldConds, cti$scores, y, freqs, def = as.integer(conCovDef))
+    if (any(isSuff <- cons[1, ] >= control$con.msc, na.rm = TRUE)){
       isSuff[is.na(isSuff)] <- FALSE
       minSuff[[.step]] <- structure(allkfoldConds[isSuff, , drop = FALSE], 
                                     conCov = cons[, isSuff, drop = FALSE])
-      if (!only.minimal.msc) attr(minSuff[[.step]], "minimal") <- minimList[[.step]][isSuff]
+      if (!control$only.minimal.msc) 
+        attr(minSuff[[.step]], "minimal") <- minimList[[.step]][isSuff]
     }  
   }
   minSuff
 }
-make.msc <- function(x, outcome, cti, details, suff.only){
+
+make.msc <- function(x, outcome, cti, details){
   if (is.null(x)) return(NULL)
   vnms <- colnames(cti$scores)
   out <- data.frame(
@@ -248,28 +220,24 @@ make.msc <- function(x, outcome, cti, details, suff.only){
     condition = C_mconcat(C_relist_Char(vnms[t(x)], 
                                         rep(ncol(x), nrow(x))), 
                           sep = "*"),
-    setNames(data.frame(t(attr(x, "conCov"))), c("consistency", "coverage")),
+    setNames(data.frame(t(attr(x, "conCov"))), c("con", "cov")),
     stringsAsFactors = FALSE)
-  # if (cti$type == "mv"){
-  #   out <- out[out$outcome != out$condition, , drop = FALSE]
-  #   if (nrow(out) == 0) return(NULL)
-  # }
+  nstars <- gregexpr("*", out$condition, fixed = TRUE)
+  out$complexity <- lengths(nstars) + 1L - (vapply(nstars, "[[", integer(1), 1L) == -1L)
   if (!is.null(attr(x, "minimal"))){
     out$minimal <- attr(x, "minimal")
-  } else {
-    out$minimal <- TRUE
   }
-  details <- setdiff(details, "inus")
-  if (length(details)){
-    out[details] <- .det.cti(cti, paste0(out$condition, "->", out$outcome), 
-                             what = details)
+  if (anyDet(details)){
+    out[getDetailCols(details)] <- .det(cti, paste0(out$condition, "->", out$outcome), 
+                                        what = details$detailMeasures, 
+                                        conCovMeasures = details$conCovMeasures)
   }
   rownames(out) <- NULL
   out
 }
 
-findAsf <- function(cti, y, freqs, con, cov, .conjList, maxSol, maxstep, only.minimal.asf, 
-                    verbose = FALSE){
+findAsf <- function(cti, y, freqs, con, cov, .conjList, maxstep, control, 
+                    conCovDef = 1:2, verbose = FALSE){
   .conSc <- lapply(seq_along(.conjList), function(i) C_conjScore(cti$scores, .conjList[[i]]))
   maxstep1 <- maxstep
   maxstep1[[1]] <- min(maxstep1[[1]], length(.conSc))
@@ -293,8 +261,8 @@ findAsf <- function(cti, y, freqs, con, cov, .conjList, maxSol, maxstep, only.mi
     }
     .newsol <- list()
     if (prod(nn[.c]) > 0 && all(tabulate(.c, nbins = length(.conSc)) <= nn)){
-      
-      .s <- C_find_asf(.c, .conSc[.c], y, freqs, con, cov, maxSol)
+      .s <- C_find_asf(.c, .conSc[.c], y, freqs, con, cov, 
+                       maxSol = control$maxSol, def = conCovDef)
       if (nrow(.s) > 0){
         .cands <- lapply(seq_along(.c), function(i){
           .part <- .conjList[[.c[i]]][.s[, i] + 1L, , drop = FALSE]
@@ -302,8 +270,8 @@ findAsf <- function(cti, y, freqs, con, cov, .conjList, maxSol, maxstep, only.mi
         })
         .newsol <- do.call(mapply, c(list(list), .cands, list(SIMPLIFY = FALSE)))
         stopifnot(is.recIntList(.newsol))
-        if (only.minimal.asf && length(.sol)){ 
-          .newsol <- .newsol[C_minimal_old(.newsol, .sol, ignore_equals = FALSE)]
+        if (control$only.minimal.asf && length(.sol)){ 
+          .newsol <- .newsol[C_minimal(.newsol, .sol, strict = TRUE)]
         }
         .sol <- c(.sol, .newsol)
       }
@@ -321,32 +289,43 @@ findAsf <- function(cti, y, freqs, con, cov, .conjList, maxSol, maxstep, only.mi
   if (verbose) cat("\r                                              \r")
   .sol
 }
-make.asf <- function(cti, zname, .sol, inus.only, details, 
-                     asf.selection, verbose = FALSE){
+make.asf <- function(cti, zname, .sol, control, details, conCovDef = 1:2, 
+                     verbose = FALSE){
   vnms <- colnames(cti$scores)
   .lhs <- hconcat(.sol, c("+", "*"), f = function(i) vnms[i])
-  asf <- qcondTbl_asf(paste0(.lhs, "<->", zname), cti$scores, cti$freq, 
-                      asf.selection = asf.selection)
+  asf <- qcondTbl_asf(paste0(.lhs, "<->", zname), cti, 
+                      asf.selection = control$asf.selection, conCovDef = conCovDef)
+  attr(asf, "measures") <- NULL
   if (verbose && nrow(asf)<length(.lhs)){
     cat("    ", length(.lhs)-nrow(asf), " asf (of ", length(.lhs) ,
-        ") are removed based on criterion asf.selection=\"", asf.selection, "\"\n", 
-        sep = "")
+        ") are removed based on criterion asf.selection=\"", 
+        control$asf.selection, "\"\n", sep = "")
   }
   if (nrow(asf) == 0) return(NULL)
   asf$condition[] <- as.character(lhs(asf$condition))
   n_plus_stars <- gregexpr("[\\*\\+]", asf$condition)
   asf$complexity <- 
     lengths(n_plus_stars) + 1L - (vapply(n_plus_stars, "[[", integer(1), 1L) == -1L)
-  asf <- asf[order(asf$complexity, -asf$consistency * asf$coverage), , drop = FALSE]
+  if (!isTRUE(getOption("dontsort"))) 
+    asf <- asf[order(asf$complexity, -asf$con * asf$cov), , drop = FALSE]
   n0 <- nrow(asf)
-  if (length(details)){
-    .cond <- if (n0 == 0) character(0) else paste0(asf$condition, "<->", asf$outcome)
-    if (useCtiList(cti)) 
-      cti <- ctiList(cti, .cond)
-    asf <- cbind(asf, .det(cti, .cond, what = details, cycle.type = NULL))
+  if (!control$only.minimal.asf){
+    asf$minimal <- is.minimal(asf$condition)
   }
-  if (inus.only){
-    asf <- asf[asf$inus, , drop = FALSE]
+  .cond <- if (n0 == 0) character(0) else paste0(asf$condition, "<->", asf$outcome)
+  if (useCtiList(cti)) 
+    cti <- ctiList(cti, .cond)
+  
+  outCols <- union(names(asf), getDetailCols(details))
+  if (control$inus.only) details <- addDetailMeasure(details, "inus")
+  if (anyDet(details)){
+    .detCols <- getDetailCols(details) # always include "inus"
+    asf[getDetailCols(details)] <- 
+      .det(cti, .cond, what = details$detailMeasures, 
+           conCovMeasures = details$conCovMeasures)[.detCols]
+  }
+  if (control$inus.only){
+    asf <- asf[asf$inus, outCols, drop = FALSE]
     if (verbose)
       cat("    ", n0 - nrow(asf), " non-INUS asf (of ", n0 ,") are removed, as inus.only=TRUE\n", 
           sep = "")
@@ -372,27 +351,3 @@ checkOutcome <- function(x, values){
   x  
 }
 
-################################################################################
-
-# versions of cna with fixed type
-cscna <- function(...){
-  cl <- match.call(cna, sys.call())
-  stopifnot(is.null(cl$type))
-  cl[[1]] <- quote(cna)
-  cl$type <- "cs"
-  eval.parent(cl)
-}
-mvcna <- function(...){
-  cl <- match.call(cna, sys.call())
-  stopifnot(is.null(cl$type))
-  cl[[1]] <- quote(cna)
-  cl$type <- "mv"
-  eval.parent(cl)
-}
-fscna <- function(...){
-  cl <- match.call(cna, sys.call())
-  stopifnot(is.null(cl$type))
-  cl[[1]] <- quote(cna)
-  cl$type <- "fs"
-  eval.parent(cl)
-}
